import re
import math
import numpy as np
import pandas as pd

def tickGen(lo,hi,tick_num=5,offset=0):
    incr = round((hi-lo)/tick_num)
    major = np.array([r for r in range(math.ceil(lo)+offset,math.floor(hi),incr)])
    minor = np.array([r for r in range(round(lo),round(hi),1)])
    return major, minor

def unravel(ser):
    """Turns binned data into unbinned data
    ser: Pandas.Series (1D spectra)
    """
    listy=np.array([])
    for x in ser.index:
        if ser[x] > 0:
            temp = np.random.rand(1,ser[x])[0] #Produced as a nested array (1 row in the array... take singular element)
            listy = np.append(listy,temp*ser.step + x) #Linear transformation

    assert ser.sum() == len(listy), "Length of unravelled list must equal sum of counts"

    return listy

def read_1d(file):
    with open(file, "r") as a:
        soup = a.read()
    
    str = re.compile('"(.+)"')
    bin = re.compile('".+" \((-?\d+)\)')
    flt = re.compile('\((-?\d+\.?\d*)\ (-?\d+\.?\d*)\)')
    cnt = re.compile('\((\d+)\)\ (\d+)')
    
    title,label = str.findall(soup)
    bins = int(bin.findall(soup)[0])
    domain = np.array(flt.findall(soup)[0]).astype(float)
    rawdata = cnt.findall(soup)

    step = (domain[1]-domain[0])/bins
    numz = np.linspace(domain[0],domain[1]-step,bins)
    # numz = np.linspace(domain[0]+step/2,domain[1]-step/2,bins) # Data located at center of bin
    numz = numz.round(8) #Linspace can't properly round
    data = np.zeros(bins)
    for d in rawdata:
        data[int(d[0])] = int(d[1])
    ser = pd.Series(data,index=numz)
    ser = ser.astype(int)

    ser.title = title
    ser.label = label
    ser.X = [domain[0], domain[1], bins]
    ser.step = step
    ser.name='Counts'
    
    return ser

def read_2d(file):
    with open(file, "r") as a:
        soup = a.read()
    
    str = re.compile('"([^"]+)"')
    bin = re.compile('".+" +\((\d+) (\d+)\)')
    dat = re.compile('\((\d+) (\d+)\) (\d+)')
    axs = re.compile('\((-?\d+\.?\d*) (-?\d+\.?\d*)\) \((-?\d+\.?\d*) (-?\d+\.?\d*)\)')
    
    title, xlab, ylab, = str.findall(soup)
    xbin, ybin = bin.findall(soup)[0]
    xlo, xhi, ylo, yhi = axs.findall(soup)[0]
    rawdata = np.array(dat.findall(soup)).astype(int)
    
    Y = np.array([ylo,yhi,ybin]).astype(float)
    ystep = (Y[1]-Y[0])/Y[2]
    yVal = np.linspace(Y[0],Y[1],int(Y[2]))
    # yVal = np.linspace(Y[0]+ystep/2,Y[1]-ystep/2,int(Y[2])) #Shift bins by 1/2 step (for bar plotting)
    yVal = yVal.round(8)
    
    X = np.array([xlo,xhi,xbin]).astype(float)
    xstep = (X[1]-X[0])/X[2]
    xVal = np.linspace(X[0],X[1],int(X[2]))
    # xVal = np.linspace(X[0]+xstep/2,X[1]-xstep/2,int(X[2])) #Shift bins by 1/2 step (for bar plotting)
    xVal = xVal.round(8)
    
    df = pd.DataFrame(np.zeros([int(xbin),int(ybin)]),index=xVal,columns=yVal) #Create an empty frame with axes
    
    for R in rawdata:
        df.loc[xVal[R[0]],yVal[R[1]]] = R[2] #Fill in the grid
    df = df.astype(int)
    
    df.xlo=X[0]; df.xhi=X[1]; df.xbin=int(X[2]); df.xstep = xstep
    df.ylo=Y[0]; df.yhi=Y[1]; df.ybin=int(Y[2]); df.ystep = ystep
    df.title = title
    df.xlab = xlab
    df.ylab = ylab
    
    return df

##########################
### Fittable Functions ###
##########################
def line(x, m, c):
    return m * x + c

def gauss_1(x, amp, cen, sig):
    """A Standard 1-D Gaussian"""
    return amp * (1 / (sig * (np.sqrt(2 * np.pi)))) * (np.exp((-1.0 / 2.0) * (((x - cen) / sig) ** 2)))